
#REM
			           Anemometer Logger 
		 	     Version 5.0.2    24th July 2008 
		 	 		by Gill Adams

	Featuring:
	On-board logging of time and wind speed data for later transferal to PC;
	Local display;  
	User selectable wind speed unit as kph or m/s;
	User selectable sample time from 1 sec to 60 sec;
	Sleep Mode for periods of no wind.


#ENDREM		  



	#picaxe 18x	


REM 	Gill NOTE:  This sensor is pulsed 3 times per rev,with rpm to kph conversion factor of 1/5.68

	symbol pulses  = w0		; pulses from rotating head
	symbol clock   = b0		; wind speed as 'clock' pulses for display
	symbol day	   = b0		; 
	symbol temp0   = b0		; temp byte 0
	symbol date    = b1		;
	symbol temp1   = b1		; temp byte 1
	symbol dura    = b2		; duration of sample in seconds
	symbol ICn	   = b3		; eeprom IC number
	symbol adrss   = w2		; eeprom address pointer
	symbol wSpeed  = b6		; Wind Speed in kph or m/s
	symbol hrs     = b7		; hour of record
	symbol durms   = w4		; duration of sample in miliseconds 
	symbol mins	   = b8		; min of record
	symbol sec     = b9		; sec of record
	symbol lull	   = w5		; a 'dura' with no wind 	
	symbol temp2   = b10		; temp byte 2	
	symbol lull_10m = w6		; lull count for 10min
	symbol month   = b12		; 
	symbol year	   = b13		;


	EEPROM 0, ("Sat")
	EEPROM 3, ("Sun")
	EEPROM 6, ("Mon")
	EEPROM 9, ("Tue")
	EEPROM 12, ("Wed")
	EEPROM 15, ("Thu")
	EEPROM 18, ("Fri")
	EEPROM 21, ("kph")
	EEPROM 24, ("m/s")
	;eeprom 27 				; duration of sample in seconds'dura' _ saved
	;eeprom 28				; code for speed 'unit' _ saved
	;eeprom 29, 30			; 'lull' _ saved

;@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ Setup Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@
	setint %00000001, %00000001	
Setup:
	high 5				; turn display _ ON
	read 27, dura
	if dura = 0 then			; has a sample duration been set yet? 
	  dura = 3				; make 3sec the sample duration (default)
	  write 27, dura			; save to eeprom
	endif
	lull = 0				; clear old values
	write 29, word lull		; clear old values
	ICn = %10100000			; sets to first eeprom (IC 0)
	pulsout 7,10 			; zero the display
	adrss = 0				; set address pointer to 0
	gosub Beep				; sound "OK"
	
;@@@@@@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ Meter Wind Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@@@@@@
Measure:
	low 3					; green LED _ OFF					
	if pin6 = 1 then Adjust		; is 'Adjust sample duration' button_ ON?
	if pin2 = 1 then SetUp_DL	; is 'Down Load Data' button_ ON?
	durms = dura * 1000		; calc miliseconds for sample
	count 7, durms, pulses		; count pulses from anemometer over 'dura' miliseconds 
	if pin0 = 1 then					; is M/s switch_ ON ?
	  pulses = pulses*25/dura*10/36*100/71	; convert pulses to 1/10m/sec	(max 90kph or 25.5m/s) 
	else
	  pulses = pulses*25/dura*10/71	; convert pulses to kph       (max 150kph)
	endif
	let wSpeed = pulses			; extract data variable 'wSpeed' from word variable 'pulses'
	
;@@@@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ Log Data Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@@@@
LogWhat:
	i2cslave %11010000, i2cslow,i2cbyte	; address the clock IC
	if ICn = %10110000 then PrePlay 	; are all eeprom ICs_ FULL?	
	if pin1 = 0 then				; is logging switch_ OFF ?
	  if adrss > 0 then Close
	  goto PrePlay
	endif
	if adrss = 32767 then			; is this current eeprom IC_ FULL?
	  ICn = ICn + 2				; set for next IC (ICn allocations increment x 2)
	  adrss = 0 
	endif
	if wSpeed < 2 then NoWind		; is there a wind lull?
	lull = 0
PreLog:
	high 3					; green LED _ ON showing data is being saved
	i2cslave %11010000, i2cslow,i2cbyte	; address the clock IC
	pause 10
	readi2c 0, (sec,mins,hrs)		; read clock
	if hrs = $23 and mins > $48 then	; when approaching new day _ disable sleep mode
	  if lull > 1 then		
	    write 29, word lull			; save 'lull' count to eeprom
	    pause 10
	    lull = 0 				; set 'lull' furthest from entering sleep mode
	  else 
	    lull = 0				; and keep 'lull' below 2
	  endif	
	endif
	if ICn = %10100000 and adrss = 0 then DateMe	; date the start of data
	if hrs = $0 and mins = $0 then
	  temp1 = sec & %11110000 / 16 * 10		; convert BCD to Decimal for seconds
	  temp1 = sec & %00001111 + temp1		; convert BCD to Decimal for seconds
	  if temp1 < dura then DateMe 		; if a new day is next _ then date it
	  endif
	if ICn = %10101110 and adrss > 32761 then DateMe	; date the end of data
	i2cslave ICn, i2cfast,i2cword					; address the eeprom ICs
Log:
	writei2c adrss, (hrs,mins)		; write to eeprom IC
	pause 10
	adrss = adrss + 2 max 32767		; set next address  
	writei2c adrss, (sec,wSpeed)		; write to eeprom IC
	pause 10
	adrss = adrss + 2 max 32767		; set next address
	goto PrePlay

DateMe:
	readi2c 0, (sec,mins,hrs,day,date,month,year)	; read clock
	i2cslave ICn, i2cfast,i2cword		; address the eeprom ICs
	writei2c adrss, (day,date) 		; write to eeprom IC
	pause 10
	adrss = adrss + 2 max 32767		; set next address
	writei2c adrss, (month,year)		; write to eeprom IC
	pause 10
	adrss = adrss + 2 max 32767		; set next address
	read 29, word lull			; new day now dated so back to former 'lull' count
	if adrss = 32767 then LogWhat		; is this current eeprom IC full?
	if pin1 = 0 then Setup 			; is logging switch_ OFF ?  (dating end of data)
	goto LOG
	
LogS:
	i2cslave %11010000, i2cslow,i2cbyte	; address the clock IC
	readi2c 0, (sec,mins,hrs)		; read clock
	i2cslave ICn, i2cfast,i2cword		; address the eeprom ICs
	pause 10
	writei2c adrss, (hrs,mins)		; write to eeprom IC
	pause 10
	adrss = adrss + 2 max 32767		; set next address
	writei2c adrss, (sec,233)		; write a marker(233) for sleep mode to eeprom IC 
	pause 10
	adrss = adrss + 2 max 32767		; set next address
	return

Close:
	i2cslave ICn, i2cfast,i2cword		; address the eeprom IC
	writei2c adrss, (244, 244) 		; write an end of data marker(244) to eeprom IC
	pause 10
	adrss = adrss + 2 max 32767		; set next address
	writei2c adrss, (244, 244) 		; write another end of data marker to eeprom IC
	pause 10
	adrss = adrss + 2 max 32767		; set next address
	i2cslave %11010000, i2cslow,i2cbyte	; address the clock IC
	goto DateMe

;@@@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ Display Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@@@
PrePlay:					; do the "return"/"goto" double shuffle
	gosub Display
	goto Measure	
Display: 
	pulsout 7,10			; reset the display
	if wSpeed = 0 then return: endif
	FOR clock = 1 to wSpeed		; set for a qty of clock pulses = WSpeed
	  pulsout 6,10 			; pulse the display IC's clock pin
	NEXT	 				; next loop 
	return

;@@@@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ DownLoad Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@@@@
SetUp_DL:					; download all data to PC
	gosub Beep				; sound "OK"
	high 2				; red LED _ ON
	low 5					; turn display _ OFF
	ICn = %10100000			; set for 1st eeprom IC
	adrss = 0				; set for 1st address
	high 0: pause 100			; progress beeps of the next eeprom IC
	low 0: pause 200
	read 28, temp1			; read in code for unit of measurement 
	branch temp1,(KmpH, MtrS)
KmpH:
	sertxd (009,009,009,009,"avg kph for a ",#dura,"sec sample",cr,lf)	; data to PC
	goto DwnLoad
MtrS:	
	sertxd (009,009,009,009,"avg m/s for a ",#dura,"sec sample",cr,lf)	; data to PC
	
	
DwnLoad:
	i2cslave ICn, i2cfast,i2cword					; address eeprom IC
	if ICn = %10100000 and adrss = 0 then gosub DoDate	; date start of data 
	temp0 = 60 - dura
	DO
	  if ICn = %10101110 and adrss > 32761 then Dl_Over	; date end of data
	  readi2c adrss, (hrs,mins)
	  adrss = adrss + 2 max 32767	
	  readi2c adrss, (sec,wSpeed)
	  adrss = adrss + 2 max 32767
	  if mins = 244 or wSpeed = 244 then Dl_Over	; look for end of data marker (244)
	  gosub BCD_Decimal2					; convert the clocks BCD  to decimal
	  sertxd (009,009,#hrs,":",#mins,":",#sec," _ ")
	  if wSpeed = 233 then		; is there a sleep marker (233)
	    sertxd ("0 s",cr,lf)
	  else
	    sertxd (#wSpeed,cr,lf)	; data to PC
	  endif
	  if hrs = 23 and mins = 59 and sec => temp0 then gosub DoDate	; is a new day next?
	LOOP until adrss = 32767	; loop until this IC emptied
	goto NextIC

NextIC:
	ICn = ICn + 2			; set for next IC number
	adrss = 0				; set for 1st address
	temp1 = ICn + 2
	DO
	  high 0: pause 100		; progress beeps of the next eeprom IC
	  low 0: pause 200
	  temp1 = temp1 - 2
	LOOP until temp1 = %10100000
	goto DwnLoad
	
DoDate:
	readi2c adrss, (day,date)			
	pause 10
	adrss = adrss + 2 max 32767		
	readi2c adrss, (month,year)
	pause 10
	adrss = adrss + 2 max 32767		
	gosub BCD_Decimal1
	sertxd ("  ")			; data to PC
	branch day, (Sat, Sun, Mon, Tue, Wed, Thu, Fri)	; convert 'day' code to text
Sat:	
	FOR temp2 = 0 to 2
        read temp2, day			; read PicAXE eeprom
        sertxd (day)			; send "Sat"
      NEXT
      goto MoreDate
Sun:	
	FOR temp2 = 3 to 5
        read temp2, day			; read PicAXE eeprom
        sertxd (day)			; send "Sun"
      NEXT
      goto MoreDate
Mon:	
	FOR temp2 = 6 to 8
        read temp2, day			; read PicAXE eeprom
        sertxd (day)			; send "Mon"
      NEXT
      goto MoreDate
Tue:	
	FOR temp2 = 9 to 11
        read temp2, day			; read PicAXE eeprom
        sertxd (day)			; send "Tue"
      NEXT
      goto MoreDate
Wed:	
	FOR temp2 = 12 to 14
        read temp2, day			; read PicAXE eeprom
        sertxd (day)			; send "Wed"
      NEXT
      goto MoreDate
Thu:	
	FOR temp2 = 15 to 17
        read temp2, day			; read PicAXE eeprom
        sertxd (day)			; send "Thu"
      NEXT
      goto MoreDate
Fri:	
	FOR temp2 = 18 to 20
        read temp2, day			; read PicAXE eeprom
        sertxd (day)			; send "Fri"
      NEXT
      
MoreDate:
	sertxd (" ",#date,"/",#month,"/",#year,cr,lf)	; data to PC
	temp0 = 60 - dura
	return
	
Dl_Over:
	gosub DoDate		
	sertxd (cr,lf,"--END--")	; data to PC for Terminal
	gosub Beep				; download over _ beep 3x OK's
  	gosub Beep
	low 2					; red LED _ OFF
	goto Setup				; and back to reSetup

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ Conversion Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@@@@@@
BCD_Decimal1:
	temp2 = day & %11110000 / 16 * 10
	day = day & %00001111 + temp2
	temp2 = date & %11110000 / 16 * 10
	date = date & %00001111 + temp2
	temp2 = month & %11110000 / 16 * 10
	month = month & %00001111 + temp2
	temp2 = year & %11110000 / 16 * 10
	year = year & %00001111 + temp2
	return
BCD_Decimal2:
	temp1 = sec & %11110000 / 16 * 10
	sec = sec & %00001111 + temp1
	temp1 = mins & %11110000 / 16 * 10
	mins = mins & %00001111 + temp1
	temp1 = hrs & %11110000 / 16 * 10
	hrs = hrs & %00001111 + temp1
	return

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ Change Metering Unit Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
interrupt:
	gosub Beep				; sound "OK"
	if pin0 = 1 then gosub M_s	; Which unit is selected ?
	if pin0 = 0 then gosub KPH
	goto Setup
KPH:
	temp1 = 0
	write 28, temp1
	setint %00000001, %00000001	; now look for a change to M-s
	return	
M_s:
	temp1 = 1
	write 28, temp1
	setint %00000000, %00000001	; now look for a change to KPH
	return

;@@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ Sounds Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@@
Beep:						; OK
	high 0: pause 80
	low 0: pause 20
	high 0: pause 80			; RoadRunner's bee-beep
	low 0: pause 500
	return

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ Set Sample Time Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
Adjust:					; adjust the duration of wind speed sample
	gosub Beep
	pulsout 7,10 			; zero the display
	pause 500
	dura = 0
	DO until dura = 60		; 60sec max
	  inc dura
	  let wSpeed = dura		; make 'wSpeed' a proxy for 'dura' so to fit with Display code		
	  gosub Display			; display 'dura'
	  high 0:pause 80			; lone beep sets an audio beat for easy timing of button push
	  low 0:pause 800
	LOOP until pin6 = 1		; button pushed _ new duration accepted
	write 27, dura			; save new value to picaxe eeprom	
	goto Setup

;@@@@@@@@@@@@@@@@@@@@@@@
;@@@@ Sleep Routine @@@@
;@@@@@@@@@@@@@@@@@@@@@@@
NoWind:
	lull_10m = 600 / dura		; calc lull count that makes 10min(600sec)
	if lull = lull_10m then Dozzze	; if no wind for initial 10 min or 1 min thereafter, have 10 min Doze
	inc lull				; advance the lull count by 1
	goto PreLog 
Dozzze:
	gosub LogS
	low 5					; turn display _ OFF
	sleep 260				; enter sleep mode for 10 minutes
 	let lull = 540 / dura		; set lull to a count = to 9min(540sec)
	high 5				; turn display _ ON
	goto Measure
	

	
	
END


#REM
 
 Version 5.0.2 :-
 Delete logging the Date on change to next eeprom IC
 Log the Date at start of logging, start of a new day, and end of logging.

 Version 5.0.1 :-
 Records all Zero wind speeds.
 Tags the 10min Sleep period with a "Sleep" entry.
 Windmill logger code removed
 Downloads data in a more MS Excel friendly format (I hope)

#ENDREM













































